

import re

from jk_cachefunccalls import cacheCalls

from .parsing_utils import *
from .invoke_utils import run



#
# Returns:
#
#	{
#		"/sys": {
#				"dev": null,
#				"fstype": "sysfs",
#				"fstype2": "sysfs",
#				"mountPoint": "/sys",
#				"options": [
#						"rw",
#						"nosuid",
#						"nodev",
#						"noexec",
#						"relatime"
#				]
#		},
#		"/proc": {
#				"dev": null,
#				"fstype": "proc",
#				"fstype2": "proc",
#				"mountPoint": "/proc",
#				"options": [
#						"rw",
#						"nosuid",
#						"nodev",
#						"noexec",
#						"relatime"
#				]
#		},
#		"/dev": {
#				"dev": null,
#				"fstype": "devtmpfs",
#				"fstype2": "udev",
#				"mountPoint": "/dev",
#				"options": [
#						"rw",
#						"nosuid",
#						"relatime",
#						"size=15913668k",
#						"nr_inodes=3978417",
#						"mode=755"
#				]
#		},
#		...,
#		"/run": {
#				"dev": null,
#				"fstype": "tmpfs",
#				"fstype2": "tmpfs",
#				"mountPoint": "/run",
#				"options": [
#						"rw",
#						"nosuid",
#						"noexec",
#						"relatime",
#						"size=3187068k",
#						"mode=755"
#				]
#		},
#		...,
#		"/mounts/net/nbxxxxxxxx": {
#				"dev": "xxxxxxxx@192.168.10.16:/home/xxxxxxxx",
#				"fstype": "fuse.sshfs",
#				"fstype2": null,
#				"mountPoint": "/mounts/net/nbxxxxxxxx",
#				"options": [
#						"rw",
#						"nosuid",
#						"nodev",
#						"relatime",
#						"user_id=1000",
#						"group_id=1000",
#						"allow_other"
#				]
#		},
#		...
#	}
#
def parse_mount(stdout:str, stderr:str, exitcode:int) -> dict:

	"""
	sysfs on /sys type sysfs (rw,nosuid,nodev,noexec,relatime)
	proc on /proc type proc (rw,nosuid,nodev,noexec,relatime)
	udev on /dev type devtmpfs (rw,nosuid,relatime,size=8018528k,nr_inodes=2004632,mode=755)
	devpts on /dev/pts type devpts (rw,nosuid,noexec,relatime,gid=5,mode=620,ptmxmode=000)
	tmpfs on /run type tmpfs (rw,nosuid,noexec,relatime,size=1608252k,mode=755)
	/dev/sdb1 on / type ext4 (rw,relatime,errors=remount-ro,data=ordered)
	...
	"""

	if exitcode != 0:
		raise Exception()

	lines = stdout.strip().split("\n")

	ret = {}
	for line in lines:
		m = re.match(r"^(.+?) on (.+?) type ([^\s]+) \(([^\s]+)\)$", line)
		if m is None:
			raise Exception("Failed to parse line: " + repr(line))

		groups = m.groups()
		devicePathOrFileSystem = groups[0]
		mountPoint = groups[1]
		fstype = groups[2]
		mountOptions = groups[3].split(",")
		isDevicePath = devicePathOrFileSystem.find("/") >= 0
		devicePath = devicePathOrFileSystem if isDevicePath else None
		fstype2 = None if isDevicePath else devicePathOrFileSystem
		ret[mountPoint] = {
			"dev": devicePath,
			"fstype": fstype,
			"fstype2": fstype2,
			"options": mountOptions,
			"mountPoint": mountPoint,
		}

	return ret
#



#
# Returns:
#
#	{
#		"/sys": {
#				"dev": null,
#				"fstype": "sysfs",
#				"fstype2": "sysfs",
#				"mountPoint": "/sys",
#				"options": [
#						"rw",
#						"nosuid",
#						"nodev",
#						"noexec",
#						"relatime"
#				]
#		},
#		"/proc": {
#				"dev": null,
#				"fstype": "proc",
#				"fstype2": "proc",
#				"mountPoint": "/proc",
#				"options": [
#						"rw",
#						"nosuid",
#						"nodev",
#						"noexec",
#						"relatime"
#				]
#		},
#		"/dev": {
#				"dev": null,
#				"fstype": "devtmpfs",
#				"fstype2": "udev",
#				"mountPoint": "/dev",
#				"options": [
#						"rw",
#						"nosuid",
#						"relatime",
#						"size=15913668k",
#						"nr_inodes=3978417",
#						"mode=755"
#				]
#		},
#		...,
#		"/run": {
#				"dev": null,
#				"fstype": "tmpfs",
#				"fstype2": "tmpfs",
#				"mountPoint": "/run",
#				"options": [
#						"rw",
#						"nosuid",
#						"noexec",
#						"relatime",
#						"size=3187068k",
#						"mode=755"
#				]
#		},
#		...,
#		"/mounts/net/nbxxxxxxxx": {
#				"dev": "xxxxxxxx@192.168.10.16:/home/xxxxxxxx",
#				"fstype": "fuse.sshfs",
#				"fstype2": null,
#				"mountPoint": "/mounts/net/nbxxxxxxxx",
#				"options": [
#						"rw",
#						"nosuid",
#						"nodev",
#						"relatime",
#						"user_id=1000",
#						"group_id=1000",
#						"allow_other"
#				]
#		},
#		...
#	}
#
@cacheCalls(seconds=3, dependArgs=[0])
def get_mount(c = None) -> dict:
	stdout, stderr, exitcode = run(c, "/bin/mount")
	return parse_mount(stdout, stderr, exitcode)
#























import datetime
import pytz
from dateutil import tz

from jk_cachefunccalls import cacheCalls

from .parsing_utils import *
from .invoke_utils import run
from .get_date import *



_EPOCH = datetime.datetime(1970, 1, 1, 0, 0, 0, 0, pytz.UTC)




#
#
#
def parse_uptime(stdout:str, stderr:str, exitcode:int) -> float:

	"""
	395207.66 1784474.45
	"""

	if exitcode != 0:
		raise Exception()

	sUptime, _ = stdout.strip().split(" ")
	uptimeInSeconds = float(sUptime)

	return uptimeInSeconds
#



#
# Returns:
#
#	{
#		"systemNow": 1567404690.778003,
#		"systemNowLocal": {
#				"day": 2,
#				"hour": 9,
#				"millisecond": 778,
#				"minute": 11,
#				"month": 9,
#				"second": 30,
#				"year": 2019
#		},
#		"systemNowUTC": {
#				"day": 2,
#				"hour": 6,
#				"millisecond": 778,
#				"minute": 11,
#				"month": 9,
#				"second": 30,
#				"year": 2019
#		},
#		"systemStartTime": 1566972543.348003,
#		"systemStartTimeLocal": {
#				"day": 28,
#				"hour": 9,
#				"millisecond": 348,
#				"minute": 9,
#				"month": 8,
#				"second": 3,
#				"year": 2019
#		},
#		"systemStartTimeUTC": {
#				"day": 28,
#				"hour": 6,
#				"millisecond": 348,
#				"minute": 9,
#				"month": 8,
#				"second": 3,
#				"year": 2019
#		},
#		"uptimeInSeconds": 432147.43
#	}
#
def get_uptime(c = None) -> dict:
	systemNowInDateTimeUTC = get_date_as_datetime(None, True)
	systemNowInSeconds = (systemNowInDateTimeUTC - _EPOCH).total_seconds()
	systemNowInDateTimeLocal = systemNowInDateTimeUTC.astimezone(tz.tzlocal())

	stdout, stderr, exitcode = run(c, "cat /proc/uptime")
	uptimeInSeconds = parse_uptime(stdout, stderr, exitcode)

	systemStartInSeconds = systemNowInSeconds - uptimeInSeconds
	systemStartInDateTimeUTC = datetime.datetime.utcfromtimestamp(systemStartInSeconds).replace(tzinfo=pytz.UTC)
	systemStartInDateTimeLocal = systemStartInDateTimeUTC.astimezone(tz.tzlocal())

	ret = {
		"systemNow": systemNowInSeconds,
		"systemNowUTC": {
			"year": systemNowInDateTimeUTC.year,
			"month": systemNowInDateTimeUTC.month,
			"day": systemNowInDateTimeUTC.day,
			"hour": systemNowInDateTimeUTC.hour,
			"minute": systemNowInDateTimeUTC.minute,
			"second": systemNowInDateTimeUTC.second,
			"millisecond": systemNowInDateTimeUTC.microsecond // 1000,
		},
		"systemNowLocal": {
			"year": systemNowInDateTimeLocal.year,
			"month": systemNowInDateTimeLocal.month,
			"day": systemNowInDateTimeLocal.day,
			"hour": systemNowInDateTimeLocal.hour,
			"minute": systemNowInDateTimeLocal.minute,
			"second": systemNowInDateTimeLocal.second,
			"millisecond": systemNowInDateTimeLocal.microsecond // 1000,
		},
		"systemStartTime": systemStartInSeconds,
		"systemStartTimeUTC": {
			"year": systemStartInDateTimeUTC.year,
			"month": systemStartInDateTimeUTC.month,
			"day": systemStartInDateTimeUTC.day,
			"hour": systemStartInDateTimeUTC.hour,
			"minute": systemStartInDateTimeUTC.minute,
			"second": systemStartInDateTimeUTC.second,
			"millisecond": systemStartInDateTimeUTC.microsecond // 1000,
		},
		"systemStartTimeLocal": {
			"year": systemStartInDateTimeLocal.year,
			"month": systemStartInDateTimeLocal.month,
			"day": systemStartInDateTimeLocal.day,
			"hour": systemStartInDateTimeLocal.hour,
			"minute": systemStartInDateTimeLocal.minute,
			"second": systemStartInDateTimeLocal.second,
			"millisecond": systemStartInDateTimeLocal.microsecond // 1000,
		},
		"uptimeInSeconds": uptimeInSeconds,
	}

	return ret
#




















from jk_cachefunccalls import cacheCalls

from .parsing_utils import *
from .invoke_utils import run






#
# Returns:
#	{
#		"name": "B150M ECO (MS-7994)",
#		"vendor": "MSI",
#		"version": "1.0"
#	}
#
@cacheCalls(seconds=3, dependArgs=[0])
def get_motherboard_info(c = None) -> dict:
	bFail = False

	try:
		# TODO: sysinfo.py avoids calling this method on Raspberry Pi devices thus working around getting the desired information this way. 
		model, _, _ = run(c, "cat /sys/firmware/devicetree/base/model")
		PATTERN = "Raspberry Pi"
		if model.startswith(PATTERN):
			return {
				"vendor": "Raspberry Pi Foundation",
				"name": "Raspberry Pi",
				"version": model[len(PATTERN):].strip(),
			}
		else:
			print(repr(model))
			bFail = True
	except:
		try:
			vendor, _, _ = run(c, "cat /sys/devices/virtual/dmi/id/board_vendor")
			name, _, _ = run(c, "cat /sys/devices/virtual/dmi/id/board_name")
			version, _, _ = run(c, "cat /sys/devices/virtual/dmi/id/board_version")

			return {
				"vendor": vendor.strip(),
				"name": name.strip(),
				"version": version.strip(),
			}
		except:
			bFail = True

	if bFail:
		return {}
		#raise Exception("Unknown system layout: Can't retrieve required information!")
#












import re

from jk_cachefunccalls import cacheCalls

from .parsing_utils import *
from .invoke_utils import run



_parserColonKVP = ParseAtFirstDelimiter(delimiter=":", valueCanBeWrappedInDoubleQuotes=False, keysReplacesSpacesWithUnderscores=False)




#
# Note: This function is a building block for <c>get_net_info()</c>.
#
# Returns:
#	{
#		"br0": {
#			"hwaddr": "d8:cb:8a:ec:5f:05",
#			"ifname": "br0",
#			"ip4_addr": "192.168.10.10",
#			"ip4_broadcast_addr": "  Bcast:192.168.255.255",
#			"ip4_netmask": "255.255.0.0",
#			"ip6_addr": "fe80::dacb:8aff:feec:5f05/64",
#			"ip6_scope": "Link",
#			"metric": 1,
#			"mtu": 1500,
#			"rx_bytes": 11664556911,
#			"rx_dropped": 5938,
#			"rx_errors": 0,
#			"rx_frame": 0,
#			"rx_overruns": 0,
#			"rx_packetCount": 9803375,
#			"tx_bytes": 1178570750,
#			"tx_carrier": 0,
#			"tx_dropped": 0,
#			"tx_errors": 0,
#			"tx_overruns": 0,
#			"tx_packetCount": 7143430,
#			"tx_queuelen": "1000",
#			"type": "Ethernet"
#		},
#		"enp0s31f6": {
#			"hwaddr": "d8:cb:8a:ec:5f:05",
#			"ifname": "enp0s31f6",
#			"ip6_addr": "fe80::dacb:8aff:feec:5f05/64",
#			"ip6_scope": "Link",
#			"memory": "df000000-df020000",
#			"metric": 1,
#			"mtu": 1500,
#			"rx_bytes": 11894173175,
#			"rx_dropped": 0,
#			"rx_errors": 0,
#			"rx_frame": 0,
#			"rx_overruns": 0,
#			"rx_packetCount": 10302195,
#			"tx_bytes": 1225779865,
#			"tx_carrier": 0,
#			"tx_dropped": 0,
#			"tx_errors": 0,
#			"tx_overruns": 0,
#			"tx_packetCount": 7384311,
#			"tx_queuelen": "1000",
#			"type": "Ethernet"
#		},
#		"lo": {
#			"hwaddr": null,
#			"ifname": "lo",
#			"ip4_addr": "127.0.0.1",
#			"ip4_broadcast_addr": null,
#			"ip4_netmask": "255.0.0.0",
#			"ip6_addr": "::1/128",
#			"ip6_scope": "Host",
#			"metric": 1,
#			"mtu": 65536,
#			"rx_bytes": 6132108,
#			"rx_dropped": 0,
#			"rx_errors": 0,
#			"rx_frame": 0,
#			"rx_overruns": 0,
#			"rx_packetCount": 5508,
#			"tx_bytes": 6132108,
#			"tx_carrier": 0,
#			"tx_dropped": 0,
#			"tx_errors": 0,
#			"tx_overruns": 0,
#			"tx_packetCount": 5508,
#			"tx_queuelen": "1",
#			"type": "Local Loopback"
#		}
#	}
#
def parse_ifconfig(stdout:str, stderr:str, exitcode:int) -> dict:

	if exitcode != 0:
		raise Exception()

	"""
	enp0s31f6: flags=4099<UP,BROADCAST,MULTICAST>  mtu 1500
        ether a6:b5:c4:d3:e2:f1  txqueuelen 1000  (Ethernet)
        RX packets 0  bytes 0 (0.0 B)
        RX errors 0  dropped 0  overruns 0  frame 0
        TX packets 0  bytes 0 (0.0 B)
        TX errors 0  dropped 0 overruns 0  carrier 0  collisions 0
        device interrupt 16  memory 0xef300000-ef320000  

	lo: flags=73<UP,LOOPBACK,RUNNING>  mtu 65536
			inet 127.0.0.1  netmask 255.0.0.0
			inet6 ::1  prefixlen 128  scopeid 0x10<host>
			loop  txqueuelen 1000  (Local Loopback)
			RX packets 101707  bytes 11484587 (11.4 MB)
			RX errors 0  dropped 0  overruns 0  frame 0
			TX packets 101707  bytes 11484587 (11.4 MB)
			TX errors 0  dropped 0 overruns 0  carrier 0  collisions 0

	wlp2s0: flags=4163<UP,BROADCAST,RUNNING,MULTICAST>  mtu 1500
			inet 192.168.89.183  netmask 255.255.255.0  broadcast 192.168.89.255
			inet6 ........  prefixlen 64  scopeid 0x20<link>
			ether a1:b2:c3:d4:e5:f6  txqueuelen 1000  (Ethernet)
			RX packets 3758562  bytes 3983872988 (3.9 GB)
			RX errors 0  dropped 0  overruns 0  frame 0
			TX packets 1630817  bytes 222286497 (222.2 MB)
			TX errors 0  dropped 0 overruns 0  carrier 0  collisions 0
	"""

	lines = stdout.strip().split("\n")
	lineGroups = splitAtEmptyLines(lines)

	ret = {}
	for lineGroup in lineGroups:
		line = lineGroup[0].strip()
		m = re.match(r"^([^\s]+)\s+Link encap:(.+?)(\s+HWaddr\s([^\s]+))?$", line)
		if m is None:
			m = re.match(r"^([^\s]+):\s+flags=", line)
			if m is None:
				raise Exception("Failed to parse line: " + repr(line))
		g = m.groups()
		ifname = g[0]

		record = {
			"ifname": ifname,
			"is_loop": False,
			"mac_addr": None,
		}

		if len(g) > 1:
			record["type"] = g[1]
			record["hwaddr"] = g[3]
		ret[ifname] = record

		lineGroup = lineGroup[1:]

		for line in lineGroup:
			line = line.strip()
			if line.startswith("inet "):
				m = re.match(r"^inet addr:([^\s]+)(\s+Bcast:([^\s]+))?\s+Mask:([^\s]+)$", line)
				if m:
					g = m.groups()
					record["ip4_addr"] = g[0]
					record["ip4_broadcast_addr"] = g[2]
					record["ip4_netmask"] = g[3]
				else:
					m = re.match(r"^inet ([^\s]+)\s+netmask\s+([^\s]+)(\s+broadcast\s+([^\s]+))?$", line)
					if m:
						g = m.groups()
						record["ip4_addr"] = g[0]
						record["ip4_netmask"] = g[1]
						record["ip4_broadcast_addr"] = g[3]
					else:
						raise Exception("Failed to parse line: " + repr(line))

			elif line.startswith("inet6 "):
				m = re.match(r"^inet6 addr:\s+([^\s]+)\s+Scope:([^\s]+)$", line)
				if m:
					g = m.groups()
					record["ip6_addr"] = g[0]
					record["ip6_scope"] = g[1]
				else:
					m = re.match(r"^inet6\s+([^\s]+)\s+prefixlen\s+([^\s]+)\s+scopeid\s+([^<]+)<([^\>]+)>*$", line)
					if m:
						g = m.groups()
						record["ip6_addr"] = g[0]
						record["ip6_scope"] = g[3]
					else:
						raise Exception("Failed to parse line: " + repr(line))

			elif line.startswith("ether "):
				m = re.match(r"^ether\s+([^\s]+)\s+txqueuelen\s([^\s]+)\s+(\([^\s]+\))$", line)
				if m is None:
					raise Exception("Failed to parse line: " + repr(line))
				g = m.groups()
				record["mac_addr"] = g[0]

			elif line.startswith("loop "):
				m = re.match(r"^loop\s+txqueuelen\s+([^\s]+)\s+\(Local Loopback\)$", line)
				if m is None:
					raise Exception("Failed to parse line: " + repr(line))
				record["is_loop"] = True

			elif line.startswith("RX "):
				if line.startswith("RX packets:"):
					line = line[len("RX packets:"):]
					lineItems = line.split(" ")
					record["rx_packetCount"] = int(lineItems[0])
					for lineItem in lineItems[1:]:
						m = re.match(r"^([^\s]+):([^\s]+)$", lineItem)
						if m is None:
							raise Exception("Failed to parse line: " + repr(line))
						g = m.groups()
						record["rx_" + g[0]] = int(g[1])
				elif line.startswith("RX bytes:"):
					m = re.match(r"^RX bytes:(\d+)\s+\([^\)]+?\)\s+TX bytes:(\d+)\s+\([^\)]+?\)$", line)
					if m is None:
						raise Exception("Failed to parse line: " + repr(line))
					g = m.groups()
					record["rx_bytes"] = int(g[0])
					record["tx_bytes"] = int(g[1])
				else:
					pass

			elif line.startswith("TX "):
				if line.startswith("TX packets:"):
					line = line[len("TX packets:"):]
					lineItems = line.split(" ")
					record["tx_packetCount"] = int(lineItems[0])
					for lineItem in lineItems[1:]:
						m = re.match(r"^([^\s]+):([^\s]+)$", lineItem)
						if m is None:
							raise Exception("Failed to parse line: " + repr(line))
						g = m.groups()
						record["tx_" + g[0]] = int(g[1])
				else:
					pass

			elif line.startswith("UP "):
				lineItems = line.split("  ")
				for lineItem in lineItems[1:]:
					m = re.match(r"^([^\s]+):([^\s]+)$", lineItem)
					if m is None:
						raise Exception("Failed to parse line: " + repr(line))
					g = m.groups()
					key = g[0]
					if key == "Metric":
						key = "metric"
					elif key == "MTU":
						key = "mtu"
					else:
						raise Exception("Unknown key at " + ifname + ": " + repr(key))
					record[key] = int(g[1])

			elif line.startswith("device "):
				pass

			else:
				lineItems = line.split(" ")
				for lineItem in lineItems[1:]:
					m = re.match(r"^([^\s]+):([^\s]+)$", lineItem)
					if m is None:
						raise Exception("Failed to parse line: " + repr(line))
					g = m.groups()
					key = g[0]
					if key == "txqueuelen":
						key = "tx_queuelen"
					elif key == "Memory":
						key = "memory"
					else:
						raise Exception("Unknown key at " + ifname + ": " + repr(key))
					record[key] = g[1]

	return ret
#



#
# Returns:
#	{
#		"br0": {
#			"hwaddr": "d8:cb:8a:ec:5f:05",
#			"ifname": "br0",
#			"ip4_addr": "192.168.10.10",
#			"ip4_broadcast_addr": "  Bcast:192.168.255.255",
#			"ip4_netmask": "255.255.0.0",
#			"ip6_addr": "fe80::dacb:8aff:feec:5f05/64",
#			"ip6_scope": "Link",
#			"metric": 1,
#			"mtu": 1500,
#			"rx_bytes": 11664556911,
#			"rx_dropped": 5938,
#			"rx_errors": 0,
#			"rx_frame": 0,
#			"rx_overruns": 0,
#			"rx_packetCount": 9803375,
#			"tx_bytes": 1178570750,
#			"tx_carrier": 0,
#			"tx_dropped": 0,
#			"tx_errors": 0,
#			"tx_overruns": 0,
#			"tx_packetCount": 7143430,
#			"tx_queuelen": "1000",
#			"type": "Ethernet"
#		},
#		"enp0s31f6": {
#			"hwaddr": "d8:cb:8a:ec:5f:05",
#			"ifname": "enp0s31f6",
#			"ip6_addr": "fe80::dacb:8aff:feec:5f05/64",
#			"ip6_scope": "Link",
#			"memory": "df000000-df020000",
#			"metric": 1,
#			"mtu": 1500,
#			"rx_bytes": 11894173175,
#			"rx_dropped": 0,
#			"rx_errors": 0,
#			"rx_frame": 0,
#			"rx_overruns": 0,
#			"rx_packetCount": 10302195,
#			"tx_bytes": 1225779865,
#			"tx_carrier": 0,
#			"tx_dropped": 0,
#			"tx_errors": 0,
#			"tx_overruns": 0,
#			"tx_packetCount": 7384311,
#			"tx_queuelen": "1000",
#			"type": "Ethernet"
#		},
#		"lo": {
#			"hwaddr": null,
#			"ifname": "lo",
#			"ip4_addr": "127.0.0.1",
#			"ip4_broadcast_addr": null,
#			"ip4_netmask": "255.0.0.0",
#			"ip6_addr": "::1/128",
#			"ip6_scope": "Host",
#			"metric": 1,
#			"mtu": 65536,
#			"rx_bytes": 6132108,
#			"rx_dropped": 0,
#			"rx_errors": 0,
#			"rx_frame": 0,
#			"rx_overruns": 0,
#			"rx_packetCount": 5508,
#			"tx_bytes": 6132108,
#			"tx_carrier": 0,
#			"tx_dropped": 0,
#			"tx_errors": 0,
#			"tx_overruns": 0,
#			"tx_packetCount": 5508,
#			"tx_queuelen": "1",
#			"type": "Local Loopback"
#		}
#	}
#
@cacheCalls(seconds=3, dependArgs=[0])
def get_ifconfig(c = None) -> dict:
	stdout, stderr, exitcode = run(c, "/sbin/ifconfig -a")
	return parse_ifconfig(stdout, stderr, exitcode)
#












from jk_cachefunccalls import cacheCalls

from .parsing_utils import *
from .invoke_utils import run



#
# Returns:
#
#	{
#		"/": {
#			"dev": "/dev/sda1",
#			"fstype2": null,
#			"mountPoint": "/",
#			"spaceFree": 167776964608,
#			"spaceTotal": 984362598400,
#			"spaceUsed": 766559354880
#		},
#		"/dev": {
#			"dev": null,
#			"fstype2": "udev",
#			"mountPoint": "/dev",
#			"spaceFree": 16295596032,
#			"spaceTotal": 16295596032,
#			"spaceUsed": 0
#		},
#		...
#		"/mounts/net/nbxxxxxxxx": {
#			"dev": "xxxxxxxx@192.168.10.16:/home/xxxxxxxx",
#			"fstype2": null,
#			"mountPoint": "/mounts/net/nbxxxxxxxx",
#			"spaceFree": 137069015040,
#			"spaceTotal": 1000948457472,
#			"spaceUsed": 812962639872
#		},
#		"/mounts/ramdisk": {
#			"dev": null,
#			"fstype2": "tmpfs",
#			"mountPoint": "/mounts/ramdisk",
#			"spaceFree": 268435456,
#			"spaceTotal": 268435456,
#			"spaceUsed": 0
#		},
#		"/run": {
#			"dev": null,
#			"fstype2": "tmpfs",
#			"mountPoint": "/run",
#			"spaceFree": 3253649408,
#			"spaceTotal": 3263557632,
#			"spaceUsed": 9908224
#		},
#		...
#	}
#
def parse_df(stdout:str, stderr:str, exitcode:int) -> dict:

	"""
	Filesystem                                                                    1K-blocks        Used   Available Use% Mounted on
	udev                                                                          15913668K          0K   15913668K   0% /dev
	tmpfs                                                                          3187068K       9648K    3177420K   1% /run
	/dev/sda1                                                                    961291600K  748571804K  163866008K  83% /
	tmpfs                                                                         15935324K     652060K   15283264K   5% /dev/shm
	tmpfs                                                                             5120K          4K       5116K   1% /run/lock
	tmpfs                                                                         15935324K          0K   15935324K   0% /sys/fs/cgroup
	tmpfs                                                                           262144K          0K     262144K   0% /mounts/ramdisk
	tmpfs                                                                          3187068K        108K    3186960K   1% /run/user/1000
	"""

	if exitcode != 0:
		raise Exception()

	lines = stdout.strip().split("\n")
	candidates = getPositionsOfWords(lines[0])
	positions = []
	for i, p in enumerate(candidates):
		if i == 0:
			continue
		for j in countDownCounter(p - 1, candidates[i-1]):
			b = isVerticalSpaceColumn(lines, j)
			if b:
				positions.append(j)
				break

	ret = {}
	for line in lines[1:]:
		data = lineSplitAt(line, positions)
		devicePathOrFileSystem, sSpaceTotal, sSpaceUsed, sSpaceFree, sUsedPerCent, mountPoint = data
		if sSpaceTotal[-1] == "K":
			sSpaceTotal = sSpaceTotal[:-1]
		if sSpaceUsed[-1] == "K":
			sSpaceUsed = sSpaceUsed[:-1]
		if sSpaceFree[-1] == "K":
			sSpaceFree = sSpaceFree[:-1]
		isDevicePath = devicePathOrFileSystem.find("/") >= 0
		devicePath = devicePathOrFileSystem if isDevicePath else None
		fstype2 = None if isDevicePath else devicePathOrFileSystem
		data = {
			"dev": devicePath,
			"fstype2": fstype2,
			"spaceTotal": int(sSpaceTotal)*1024,
			"spaceUsed": int(sSpaceUsed)*1024,
			"spaceFree": int(sSpaceFree)*1024,
			"mountPoint": mountPoint,
		}
		ret[mountPoint] = data
	return ret
#



#
# Returns:
#
#	{
#		"/": {
#			"dev": "/dev/sda1",
#			"fstype2": null,
#			"mountPoint": "/",
#			"spaceFree": 167776964608,
#			"spaceTotal": 984362598400,
#			"spaceUsed": 766559354880
#		},
#		"/dev": {
#			"dev": null,
#			"fstype2": "udev",
#			"mountPoint": "/dev",
#			"spaceFree": 16295596032,
#			"spaceTotal": 16295596032,
#			"spaceUsed": 0
#		},
#		...
#		"/mounts/net/nbxxxxxxxx": {
#			"dev": "xxxxxxxx@192.168.10.16:/home/xxxxxxxx",
#			"fstype2": null,
#			"mountPoint": "/mounts/net/nbxxxxxxxx",
#			"spaceFree": 137069015040,
#			"spaceTotal": 1000948457472,
#			"spaceUsed": 812962639872
#		},
#		"/mounts/ramdisk": {
#			"dev": null,
#			"fstype2": "tmpfs",
#			"mountPoint": "/mounts/ramdisk",
#			"spaceFree": 268435456,
#			"spaceTotal": 268435456,
#			"spaceUsed": 0
#		},
#		"/run": {
#			"dev": null,
#			"fstype2": "tmpfs",
#			"mountPoint": "/run",
#			"spaceFree": 3253649408,
#			"spaceTotal": 3263557632,
#			"spaceUsed": 9908224
#		},
#		...
#	}
#
@cacheCalls(seconds=3, dependArgs=[0])
def get_df(c = None) -> dict:
	stdout, stderr, exitcode = run(c, "/bin/df -BK")
	return parse_df(stdout, stderr, exitcode)
#





















#
# This file contains various functions that assist in parsing.
# As a lot of tools require similar routines all these routines are factored out into this python file.
#


import typing



def parseKByteWithUnit(s:str):
	assert isinstance(s, str)

	if s.endswith(" KB"):
		return int(s[:-3])
	else:
		raise Exception("Can't be parsed: " + repr(s))
#



def splitAtEmptyLines(lines):
	ret = []
	buffer = []
	for line in lines:
		if line:
			buffer.append(line)
		else:
			if buffer:
				ret.append(buffer)
				buffer = []
	if buffer:
		ret.append(buffer)
		buffer = []
	return ret
#



def getPositionsOfWords(line:str) -> list:
	ret = []
	bLastWasSpace = True
	for i, c in enumerate(line):
		if c.isspace():
			bLastWasSpace = True
		else:
			if bLastWasSpace:
				ret.append(i)
			bLastWasSpace = False
	return ret
#



def isVerticalSpaceColumn(lines:str, pos:int) -> bool:
	for i in range(0, len(lines)):
		line = lines[i]
		if pos < len(line):
			if not line[pos].isspace():
				#print("@ " + str(pos) + " with " + repr(line[pos]) + " of " + repr(line))
				return False
	#print("@ " + str(pos) + " is space")
	return True
#



def splitAtVerticalSpaceColumnsFirstLineIsHeader(
	lines:str,
	expectedColumnsMin:int = -1,
	expectedColumnsMax:int = -1,
	maxColumns:int = -1
	) -> list:

	headerLine = lines[0]
	lines = lines[1:]

	positions = []
	nLastPositionWas = -99999
	for i in range(0, len(headerLine)):
		if i == 0:
			continue
		b = isVerticalSpaceColumn(lines, i)
		if b:
			if i > nLastPositionWas + 1:
				positions.append(i)
			nLastPositionWas = i
		if (maxColumns > 0) and (len(positions) == maxColumns - 1):
			break

	if expectedColumnsMin > 0:
		if (len(positions) + 1) < expectedColumnsMin:
			raise Exception("Number of columns found: " + str(len(positions)) + "; " + str(positions))

	if expectedColumnsMax > 0:
		if (len(positions) + 1) > expectedColumnsMax:
			raise Exception("Number of columns found: " + str(len(positions)) + "; " + str(positions))

	lines2 = []
	for line in lines:
		lines2.append(lineSplitAt(line, positions))

	return lines2
#



def countDownCounter(i:int, maxValue:int = 0):
	if i < maxValue:
		return
	yield i
	while True:
		i -= 1
		if i < maxValue:
			return
		yield i
#



def lineSplitAt(line:str, splitPositions:list, bTrim:bool = True):
	ret = []
	lastPos = 0
	for p in splitPositions:
		s = line[lastPos:p]
		if bTrim:
			s = s.strip()
		ret.append(s)
		lastPos = p
	s = line[lastPos:]
	if bTrim:
		s = s.strip()
	ret.append(s)
	return ret
#

def removeAllLeadingSpaces(lines:list) -> list:
	# count leading spaces
	counts = []
	for line in lines:
		for n, c in enumerate(line):
			if not c.isspace():
				if n == 0:
					# premature end: this is a line beginning with a regular character
					return lines
				counts.append(n)
				break
	minPos = min(counts)
	return [ s[minPos:] for s in lines ]
#

def groupLinesByLeadingSpace(lines:list) -> dict:
	ret = {}
	lastLineData = None
	minNumOfSpaces = 99999999999999999999
	for i, line in enumerate(lines):
		if line[0].isspace():
			numberOfSpaces = len(line) - len(line.lstrip())
			if numberOfSpaces < minNumOfSpaces:
				minNumOfSpaces = numberOfSpaces
			lastLineData.append(line)
		else:
			if minNumOfSpaces != 99999999999999999999:
				for i, s in enumerate(lastLineData):
					lastLineData[i] = s[minNumOfSpaces:]
			lastLineData = []
			ret[line] = lastLineData
			minNumOfSpaces = 99999999999999999999

	if minNumOfSpaces != 99999999999999999999:
		for i, s in enumerate(lastLineData):
			lastLineData[i] = s[minNumOfSpaces:]

	return ret
#


class ParseAtFirstDelimiter(object):

	def __init__(self, delimiter:str = ":", valueCanBeWrappedInDoubleQuotes:bool = False, keysReplacesSpacesWithUnderscores:bool = False):
		self.delimiter = delimiter
		self.keysReplacesSpacesWithUnderscores = keysReplacesSpacesWithUnderscores
		self.valueCanBeWrappedInDoubleQuotes = valueCanBeWrappedInDoubleQuotes
	#

	def parseLines(self, lines):
		assert isinstance(lines, (tuple, list))

		ret = {}
		for line in lines:
			k, v = self.parseLine(line)
			if k:
				ret[k] = v
		#
		return ret
	#

	def parseLine(self, line:str):
		assert isinstance(line, str)

		pos = line.find(self.delimiter)
		if pos > 0:
			k = line[:pos].strip()
			v = line[pos+1:].strip()
			if self.valueCanBeWrappedInDoubleQuotes:
				if v.startswith("\"") and v.endswith("\""):
					v = v[1:-1]
					v = v.replace("\\\"", "\"")
			if self.keysReplacesSpacesWithUnderscores:
				k = k.replace(" ", "_")
			return k, v
		else:
			return None, None
	#

#




def simplifyValueList(valueList:typing.Union[None,list,tuple]) -> typing.Union[None,list]:
	if valueList is None:
		return None
	assert isinstance(valueList, (tuple, list))

	# group values by unit
	temp = {}
	for item in valueList:
		assert isinstance(item, dict)
		value = item["value"]
		unit = item["unit"]
		temp2 = temp.get(unit)
		if temp2 is None:
			temp2 = []
			temp[unit] = temp2
		temp2.append(value)
	
	# unwind temp

	ret = []
	for unit, valueList in temp.items():
		if len(valueList) > 0:
			ret.append({
				"unit": unit,
				"values": valueList
			})
		else:
			ret.append({
				"unit": unit,
				"value": valueList[0]
			})

	if len(ret) == 0:
		return None
	if len(ret) == 1:
		return ret[0]
	return ret
#



def joinDictsByKey(*args):
	ret = {}
	for dictionary in args:
		assert isinstance(dictionary, dict)
		for key, d in dictionary.items():
			assert isinstance(d, dict)
			for key2, value in d.items():
				if key not in ret:
					ret[key] = {}
				ret[key][key2] = value
	return ret
#





import os
import re
import pwd
import grp

# TODO: eliminate the use of modules pwd and grp as this way get_ps() can not be executed remotely.
#		NOTE: we might be able implement this using data from get_user_info().

from jk_cachefunccalls import cacheCalls

from .parsing_utils import *
from .invoke_utils import run



_parserColonKVP = ParseAtFirstDelimiter(delimiter="=", valueCanBeWrappedInDoubleQuotes=True)



#
# Returns:
#
#	[
#			{
#					"args": "splash",
#					"cmd": "/sbin/init",
#					"pid": 1,
#					"ppid": 0,
#					"stat": "Ss",
#					"tty": null,
#					"uid": 0,
#					"user": "root"
#			},
#			{
#					"cmd": "[kthreadd]",
#					"pid": 2,
#					"ppid": 0,
#					"stat": "S",
#					"tty": null,
#					"uid": 0,
#					"user": "root",
#					"cwd": ....
#			},
#			{
#					"cmd": "[ksoftirqd/0]",
#					"pid": 3,
#					"ppid": 2,
#					"stat": "S",
#					"tty": null,
#					"uid": 0,
#					"user": "root",
#					"cwd": ....
#			},
#			...
#			{
#					"cmd": "bash",
#					"pid": 20144,
#					"ppid": 14839,
#					"stat": "Ss+",
#					"tty": "pts/3",
#					"uid": 1000,
#					"user": "xxxxxxxx"
#					"gid": 1000,
#					"group": "xxxxxxxx",
#					"cwd": ....
#			},
#			{
#					"args": "--spawner :1.9 /org/gtk/gvfs/exec_spaw/4",
#					"cmd": "/usr/lib/gvfs/gvfsd-computer",
#					"pid": 20292,
#					"ppid": 1,
#					"stat": "Sl",
#					"tty": null,
#					"uid": 1000,
#					"user": "xxxxxxxx"
#					"gid": 1000,
#					"group": "xxxxxxxx",
#					"cwd": ....
#			},
#			...
#			{
#					"args": "/usr/share/code/resources/app/extensions/json-language-features/server/dist/jsonServerMain --node-ipc --clientProcessId=15491",
#					"cmd": "/usr/share/code/code",
#					"pid": 29554,
#					"ppid": 15491,
#					"stat": "Sl",
#					"tty": null,
#					"uid": 1000,
#					"user": "xxxxxxxx"
#					"gid": 1000,
#					"group": "xxxxxxxx",
#					"cwd": ....
#			},
#			...
#	]
#
def parse_ps(stdout:str, stderr:str, exitcode:int) -> dict:
	if exitcode != 0:
		raise Exception()

	lines = stdout.strip().split("\n")
	lines2 = splitAtVerticalSpaceColumnsFirstLineIsHeader(lines, expectedColumnsMin=7, maxColumns=7)

	#	0	 1	 2		  3		 4		5	6
	# PPID   PID TT       STAT   UID   GID CMD

	ret = []
	for group in lines2:
		uid = int(group[4])
		gid = int(group[5])
		data = {
			"ppid": int(group[0]),
			"pid": int(group[1]),
			"tty": None if group[2] == "?" else group[2],
			"stat": group[3],
			"uid": uid,
			"gid": gid,
		}

		try:
			data["cwd"] = os.readlink("/proc/" + group[1] + "/cwd")
		except:
			pass

		pos = group[6].find(" ")
		if pos > 0:
			data["cmd"] = group[6][:pos]
			data["args"] = group[6][pos+1:]
		else:
			data["cmd"] = group[6]

		pwdEntry = pwd.getpwuid(uid)
		if pwdEntry:
			data["user"] = pwdEntry.pw_name

		grpEntry = grp.getgrgid(gid)
		if grpEntry:
			data["group"] = grpEntry.gr_name

		ret.append(data)

	return ret
#



#
# Returns:
#
#	[
#			{
#					"args": "splash",
#					"cmd": "/sbin/init",
#					"pid": 1,
#					"ppid": 0,
#					"stat": "Ss",
#					"tty": null,
#					"uid": 0,
#					"user": "root"
#			},
#			{
#					"cmd": "[kthreadd]",
#					"pid": 2,
#					"ppid": 0,
#					"stat": "S",
#					"tty": null,
#					"uid": 0,
#					"user": "root",
#					"cwd": ....
#			},
#			{
#					"cmd": "[ksoftirqd/0]",
#					"pid": 3,
#					"ppid": 2,
#					"stat": "S",
#					"tty": null,
#					"uid": 0,
#					"user": "root",
#					"cwd": ....
#			},
#			...
#			{
#					"cmd": "bash",
#					"pid": 20144,
#					"ppid": 14839,
#					"stat": "Ss+",
#					"tty": "pts/3",
#					"uid": 1000,
#					"user": "xxxxxxxx"
#					"gid": 1000,
#					"group": "xxxxxxxx",
#					"cwd": ....
#			},
#			{
#					"args": "--spawner :1.9 /org/gtk/gvfs/exec_spaw/4",
#					"cmd": "/usr/lib/gvfs/gvfsd-computer",
#					"pid": 20292,
#					"ppid": 1,
#					"stat": "Sl",
#					"tty": null,
#					"uid": 1000,
#					"user": "xxxxxxxx"
#					"gid": 1000,
#					"group": "xxxxxxxx",
#					"cwd": ....
#			},
#			...
#			{
#					"args": "/usr/share/code/resources/app/extensions/json-language-features/server/dist/jsonServerMain --node-ipc --clientProcessId=15491",
#					"cmd": "/usr/share/code/code",
#					"pid": 29554,
#					"ppid": 15491,
#					"stat": "Sl",
#					"tty": null,
#					"uid": 1000,
#					"user": "xxxxxxxx"
#					"gid": 1000,
#					"group": "xxxxxxxx",
#					"cwd": ....
#			},
#			...
#	]
#
# NOTE: This method makes use of the python modules <c>pwd</c> and <c>grp</c> which make use of the local user and group database only.
#		For that reason this function can not be executed remotely until the implementation has been improved here.
#
def get_ps() -> dict:
	stdout, stderr, exitcode = run(None, "ps ax -o ppid,pid,tty,stat,uid,gid,cmd")
	return parse_ps(stdout, stderr, exitcode)
#













################################################################################
################################################################################
###
###  This file is automatically generated. Do not change this file! Changes
###  will get overwritten! Change the source file for "setup.py" instead.
###  This is either 'packageinfo.json' or 'packageinfo.jsonc'
###
################################################################################
################################################################################


from setuptools import setup

def readme():
	with open("README.md", "r", encoding="UTF-8-sig") as f:
		return f.read()

setup(
	author = "Jürgen Knauth",
	author_email = "pubsrc@binary-overflow.de",
	classifiers = [
		"Development Status :: 3 - Alpha",
		"License :: OSI Approved :: Apache Software License",
		"Topic :: System :: Monitoring",
	],
	description = "This python module provides ways to retrieve and parse technical data of (Linux) computer systems.",
	download_url = "https://github.com/jkpubsrc/python-module-jk-sysinfo/tarball/0.2020.3.5",
	include_package_data = False,
	install_requires = [
		"python-dateutil",
		"fabric",
		"pytz",
		"jk_console",
		"jk_typing",
		"jk_version",
		"jk_cachefunccalls",
	],
	keywords = [
		"monitoring",
	],
	license = "Apache 2.0",
	name = "jk_sysinfo",
	packages = [
		"jk_sysinfo",
	],
	scripts = [
		"bin/sysinfo.py",
	],
	url = "https://github.com/jkpubsrc/python-module-jk-sysinfo",
	version = "0.2020.3.5",
	zip_safe = False,
	long_description = readme(),
	long_description_content_type="text/markdown",
)





from jk_cachefunccalls import cacheCalls

from .parsing_utils import *
from .invoke_utils import run



#
# Returns:
#
#	{
#		"load1": 0.13,
#		"load15": 0.29,
#		"load5": 0.3,
#		"processes_total": 1052,
#		"processes_runnable": 1
#	}
#
def parse_proc_load_avg(stdout:str, stderr:str, exitcode:int) -> dict:

	"""
	0.12 0.22 0.25 1/1048 16989
	"""

	if exitcode != 0:
		raise Exception()

	ret = stdout.strip().split(" ")
	return {
		"load1": float(ret[0]),
		"load5": float(ret[1]),
		"load15": float(ret[2]),
		"processes_runnable": int(ret[3].split("/")[0]),
		"processes_total": int(ret[3].split("/")[1]),
	}
#



#
# Returns:
#
#	{
#		"load1": 0.13,
#		"load15": 0.29,
#		"load5": 0.3,
#		"processes_total": 1052,
#		"processes_runnable": 1
#	}
#
def get_proc_load_avg(c = None) -> dict:
	stdout, stderr, exitcode = run(c, "cat /proc/loadavg")
	return parse_proc_load_avg(stdout, stderr, exitcode)
#















